<?php
/**
 * Created by JetBrains PhpStorm.
 * User: erikasouza
 * Date: 03/01/13
 * Time: 17:36
 * To change this template use File | Settings | File Templates.
 */

namespace Estoque\Logic\RecalculoEstoque;

use Estoque\Entities\Movilote;
use Estoque\Entities\Posestdia;
use Estoque\Logic\RecalculoEstoque\Exceptions\RecalculoExceptions;
use Geral\Entities\Opergrupop;
use Geral\Entities\Paramfilial;
use Geral\Entities\Posestatu;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;

class Recalculo{

    /**
     * Valida os dados informados na tela para depois executar o recálculo de estoque.
     *
     * @param $filial
     * @param $almoxarifado
     * @param $local
     * @param $produto_ini
     * @param $produto_fin
     * @param $data
     */
    public static function valida($filial, $almoxarifado, $local, $produto_ini, $produto_fin, $data){
        //produto inicial deve ser informado
        if (!$produto_ini)
            RecalculoExceptions::informarProdutoInicial();
        //produto final deve ser informado
        if (!$produto_fin)
            RecalculoExceptions::informarProdutoFinal();
        //Produto inicial deve ser menor ou igual ao produto final.
        if ($produto_ini != $produto_fin && str_pad($produto_fin, strlen($produto_ini), '0') <= $produto_ini)
            RecalculoExceptions::produtoInicialMenorIgualFinal();
        //data deve ser informada
        if (!$data)
            RecalculoExceptions::informarData();
        //para informar localização, almoxarifado deve ser informado
        if ($local && !$almoxarifado)
            RecalculoExceptions::localizacaoNaoCadastradaAlmoxarifado();

        self::validaAlmoxarifadoLocalizacaoPorOperador($filial, Ambiente::getNrorg(), $almoxarifado, $local);
    }

    /**
     * Valida se almoxarifado e localização estão habilitados para o operador logado
     */
    public static function validaAlmoxarifadoLocalizacaoPorOperador($filial, $organizacao, $almoxarifado, $local){
        $operador = Ambiente::getCdOperador();
        /** @var $paramfilial Paramfilial */
        $paramfilial  = Paramfilial::findOneByCdfilialAndNrorg($filial, $organizacao);
        $idutcacalmox = $paramfilial->getIdutcacalmox();

        //valida localização
        $utilizaLocalizacao = Opergrupop::getRepository()->localizacaoHabilitadaOperador($filial, $almoxarifado, $local,
            $operador);
        if (!$utilizaLocalizacao && $idutcacalmox == 'S')
            RecalculoExceptions::localizacaoNaoHabilitadaOperador();

        //valida almoxarifado
        $utilizaAlmoxarifado = Opergrupop::getRepository()->almoxarifadoHabilitadaOperador($filial, $almoxarifado,
            $operador);
        if (!$utilizaAlmoxarifado && $idutcacalmox == 'S')
            RecalculoExceptions::almoxarifadoNaoHabilitadoOperador();
    }

    /**
     * Recalcular o saldo do estoque.
     *
     * @param $filial
     * @param $almoxarifado
     * @param $local
     * @param $produto_ini
     * @param $produto_fin
     * @param $data
     */
    public static function recalcular($filial, $almoxarifado, $local, $produto_ini, $produto_fin, $data){
        //Apagar a posição de estoque para a filial a partir da data informada.
        Posestdia::getRepository()->deletaPosicaoEstoque($filial, $almoxarifado, $local, $produto_ini, $produto_fin,
            $data);

        //Executar a consulta e efetuar um loop com o resultado para apagar a posição do estoque:
        $produtos = Posestatu::getRepository()->consultaProdutosPosicaoEstoqueAtual($filial, $almoxarifado, $local,
            $produto_ini, $produto_fin,
            DateUtil::getDataDeString($data)->format(DateUtil::FORMATO_BRASILEIRO),
            DateUtil::adicionaIntervalo(DateUtil::getDataDeString($data))->format(DateUtil::FORMATO_BRASILEIRO));
        foreach ($produtos as $prod)
            Posestatu::getRepository()->deletaPosicaoEstoqueAtual($prod['CDFILIAL'], $prod['CDALMOXARIFE'],
                $prod['CDLOCALESTOQ'], $prod['NRLOTEESTQ'], $prod['NRSUBLOTE'], $prod['CDPRODUTO']);

        //Inserir as movimentações existentes a partir da data informada na tabela MOVILOTE
        $nrmovilote = self::novoCodigo(Ambiente::getNrorg(), $filial);
        Movilote::getRepository()->insereMovimentacoesAPartirData($filial, $almoxarifado, $local, $produto_ini,
            $produto_fin, $data, $nrmovilote);

        //Executar a procedure ATUALIZA_ESTOQUE
        self::callProcedureAtualizaEstoque($filial, $data,
            DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO), $filial, $nrmovilote);
    }

    /**
     * Gera um novo código para a movimentacao de lote
     *
     * @param $nrorg
     * @param $cdfilial
     *
     * @return int|string
     */
    public static function novoCodigo($nrorg, $cdfilial){
        $novocodigo = NovoCodigo::geraCodigo($nrorg, $cdfilial.'RECALESTOQ');
        return $novocodigo;
    }

    /**
     * Executa a procedure ATUALIZA_ESTOQUE
     *
     * @param $filimovi
     * @param $dtinicial
     * @param $dtfinal
     * @param $fililancto
     * @param $nrlancto
     *
     * @throws \Exception
     */
    public static function callProcedureAtualizaEstoque($filimovi, $dtinicial, $dtfinal, $fililancto, $nrlancto){
        try{
            $procedure = new Procedure("ATUALIZA_ESTOQUE");
            $procedure->addParam(
                new Procedure\Param(
                    'P_FILIMOVI',
                    Param::PARAM_INPUT,
                    $filimovi,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTINICIAL',
                    Param::PARAM_INPUT,
                    $dtinicial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTFINAL',
                    Param::PARAM_INPUT,
                    $dtfinal,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_FILILANCTO',
                    Param::PARAM_INPUT,
                    $fililancto,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_NRLANCTO',
                    Param::PARAM_INPUT,
                    $nrlancto,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->call();
        } catch (\Exception $e){
            throw $e;
        }
    }
}
