<?php
/**
 * Created by JetBrains PhpStorm.
 * User: victornovy
 * Date: 16/05/14
 * Time: 13:10
 * To change this template use File | Settings | File Templates.
 */

namespace Estoque\Logic\RecalculoEstoque;


use Estoque\Entities\Movilote;
use Estoque\Entities\Posestdia;
use Geral\Entities\Posestatu;
use Geral\Entities\Posestcons;
use Geral\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;

class RecalculoControleEstoque {

    /**
     * Recalcular o saldo do estoque.
     *
     * @param $filial
     * @param $produto
     * @param $produto_ini
     * @param $produto_fin
     * @param $data
     * @param $organizacao
     */
    public static function recalculo($filial, $produto, $produto_ini, $produto_fin, $data, $organizacao)
    {
        self::zeraPosicaoEstoque($filial, $produto);
        $lancamento = self::novoCodigo($organizacao, $filial);

        Movilote::getRepository()->insereMovimentacoesPorProdutoEFilial($filial, $produto_ini, $produto_fin,
            $lancamento
        );
        self::callProcedureAtualizaEstoque($filial, $data, $filial, $lancamento);
    }

    /**
     * Zerar a posicao de estoque
     *
     * @param $filial
     * @param $produto
     */
    public static function zeraPosicaoEstoque($filial, $produto)
    {
        Posestdia::getRepository()->deletaPosicaoEstoquePorProduto($filial, $produto);
        Posestcons::getRepository()->deletaPosicaoEstoquePorProduto($filial, $produto);
        Posestatu::getRepository()->deletaPosicaoEstoquePorProduto($filial, $produto);
    }

    /**
     * Gera um novo código para a movimentacao de lote
     *
     * @param $nrorg
     * @param $filial
     *
     * @return int|string
     */
    public static function novoCodigo($nrorg, $filial)
    {
        $novocodigo = NovoCodigo::geraCodigo($nrorg, $filial.'RECALESTOQ', '1', '10');
        $novocodigo = $filial.$novocodigo ;
        return $novocodigo;
    }

    /**
     * Executa a procedure ATUALIZA_ESTOQUE
     *
     * @param $filimovi
     * @param $dtfinal
     * @param $fililancto
     * @param $nrlancto
     *
     * @throws \Exception
     */
    public static function callProcedureAtualizaEstoque($filimovi, $dtfinal, $fililancto, $nrlancto)
    {
        try {
            $procedure = new Procedure("ATUALIZA_ESTOQUE");
            $procedure->addParam (
                new Procedure\Param (
                    'P_FILIMOVI',
                    Param::PARAM_INPUT,
                    $filimovi,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam (
                new Param(
                    'P_DTINICIAL',
                    Param::PARAM_INPUT,
                    NULL,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam (
                new Param(
                    'P_DTFINAL',
                    Param::PARAM_INPUT,
                    $dtfinal,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam (
                new Param(
                    'P_FILILANCTO',
                    Param::PARAM_INPUT,
                    $fililancto,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam (
                new Param(
                    'P_NRLANCTO',
                    Param::PARAM_INPUT,
                    $nrlancto,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

}